# -*- coding: utf-8 -*-
"""
/***************************************************************************
 MAGICMapLoader
                                 A QGIS plugin
 This plugin will open the DEFRA MAGIC Map service on the area of your map canvas
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2023-01-20
        git sha              : $Format:%H$
        copyright            : (C) 2023 by DeltaV Geospatial
        email                : David@DeltaVGeo.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction
from qgis.gui import QgsMessageBar

# Initialize Qt resources from file resources.py
from .resources import *
import os.path
import webbrowser


class MAGICMapLoader:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'MAGICMapLoader_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&MAGIC Map Loader')

        # Check if plugin was started the first time in current QGIS session
        # Must be set in initGui() to survive plugin reloads
        self.first_start = None

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('MAGICMapLoader', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
                
        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Adds plugin icon to Plugins toolbar
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToWebMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/magic_map_loader/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'DEFRA MAGIC Map Loader'),
            callback=self.run,
            parent=self.iface.mainWindow())

        # will be set False in run()
        self.first_start = True


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginWebMenu(
                self.tr(u'&MAGIC Map Loader'),
                action)
            self.iface.removeToolBarIcon(action)


    def run(self):
        
        # Get the bounding box from the map canvas
        canvas = self.iface.mapCanvas()
        extent = canvas.extent()

        # Check the CRS
        crs = canvas.mapSettings().destinationCrs()

        # If the CRS is not British National Grid, throw an error message and do not proceed.
        if crs.authid() != 'EPSG:27700':
            self.iface.messageBar().pushMessage("Error", "The map canvas must be in OSGB 1936 / British National Grid", level=QgsMessageBar.CRITICAL)
            return
        
        #If it is in BNG - Proceed!
        else:
            # Get the coordinates of the bounding box
            xmin = extent.xMinimum()
            ymin = extent.yMinimum()
            xmax = extent.xMaximum()
            ymax = extent.yMaximum()

            # Create the URL
            url = "https://magic.defra.gov.uk/MagicMap.aspx?&startTopic=GreyRasters&chosenLayers=baseIndex&box={}:{}:{}:{}&useDefaultbackgroundMapping=false".format(xmin, ymin, xmax, ymax)

            # Open the URL in the default browser
            webbrowser.open(url)
            
